@@ -16,9 +16,12 @@
 #' @param age.groups a character vector of age groups in increasing order.
 #' @param age.n number of months in each age groups in the same order.
 #' @param age.rw.group vector indicating grouping of the ages groups. For example, if each age group is assigned a different random walk component, then set age.rw.group to c(1:length(age.groups)); if all age groups share the same random walk component, then set age.rw.group to a rep(1, length(age.groups)). The default for 6 age groups is c(1,2,3,3,3,3), which assigns a separate random walk to the first two groups and a common random walk for the rest of the age groups. The vector should contain values starting from 1.
+#' @param age.strata.fixed.group vector indicating grouping of the ages groups for different strata. The default is c(1:length(age.groups)), which correspond to each age group within each stratum receives a separate intercept. If several age groups are specified to be the same value in this vector, the stratum specific deviation from the baseline is assumed to be the same for these age groups. For example, if \code{age.strata.fixed.group = c(1, 2, 3, 3, 3, 3)}, then the fixed effect part of the linear predictor consists of 6 overall age-specific intercepts and 3 set of strata effects (where a base stratum is chosen internally), for age groups 1, 2, and the rest respectively.
+#' 
+#' For example, if each age group is assigned a different intercept, then set age.strata.fixed.group to c(1:length(age.groups)); if all age groups share the same intercept, then set age.strata.fixed.group to a rep(1, length(age.groups)). The default for 6 age groups is the former. It can also be set to be the same as \code{age.rw.group}. The vector should contain values starting from 1.
 #' @param family family of the model. This can be either binomial (with logistic normal prior), betabiniomial.
 #' @param time.model Model for the main temporal trend, can be rw1, rw2, ar1, or NULL (for spatial-only smoothing). Default to be rw2. For ar1 main effect, a linear slope is also added with time scaled to be between -0.5 to 0.5, i.e., the slope coefficient represents the total change between the first year and the last year in the projection period on the logit scale. 
-#' @param st.time.model Temporal component model for the interaction term, can be rw1, rw2, or ar1. ar1 is not implemented for yearly model with period data input. Default to be the same as time.model unless specified otherwise. For ar1 interaction model, region-specific random slopes can be added by specifying \code{pc.st.slope.u} and \code{pc.st.slope.alpha}.
+#' @param st.time.model Temporal component model for the interaction term, can be rw1, rw2, or ar1. Default to be the same as time.model unless specified otherwise. The default does not include region-specific random slopes. They can be added to the interaction term by specifying \code{pc.st.slope.u} and \code{pc.st.slope.alpha}.  
 #' @param Amat Adjacency matrix for the regions
 #' @param bias.adj the ratio of unadjusted mortality rates or age-group-specific hazards to the true rates or hazards. It needs to be a data frame that can be merged to thee outcome, i.e., with the same column names for time periods (for national adjustment), or time periods and region (for subnational adjustment). The column specifying the adjustment ratio should be named "ratio".
 #' @param bias.adj.by vector of the column names specifying how to merge the bias adjustment to the count data. For example, if bias adjustment factor is provided in bias.adj for each region and time, then bias.adj.by should be `c("region", "time")`.
@@ -27,7 +30,8 @@
 #' @param type.st type for space-time interaction
 #' @param survey.effect logical indicator whether to include a survey iid random effect. If this is set to TRUE, there needs to be a column named 'survey' in the input data frame. In prediction, this random effect term will be set to 0.
 #' @param strata.time.effect logical indicator whether to include strata specific temporal trends.  
-#' @param common.trend logical indicator whether all age groups and/or strata share the same linear trend in the temporal main effect. Only used when the temporal main effect is an AR(1) process.
+#' @param linear.trend logical indicator whether a linear trend is added to the temporal main effect. If the temporal main effect is RW2, then it will be forced to FALSE. Default is TRUE.
+#' @param common.trend logical indicator whether all age groups and/or strata share the same linear trend in the temporal main effect.  
 #' @param hyper Deprecated. which hyperpriors to use. Only supports PC prior ("pc"). 
 #' @param pc.u hyperparameter U for the PC prior on precisions.
 #' @param pc.alpha hyperparameter alpha for the PC prior on precisions.
@@ -135,7 +139,7 @@
 #' 
 #' 
 
-smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")[1], age.groups = c("0", "1-11", "12-23", "24-35", "36-47", "48-59"), age.n = c(1,11,12,12,12,12), age.rw.group = c(1,2,3,3,3,3), time.model = c("rw1", "rw2", "ar1")[2], st.time.model = NULL, Amat, bias.adj = NULL, bias.adj.by = NULL, formula = NULL, year_label, type.st = 4, survey.effect = FALSE, common.trend = FALSE, strata.time.effect = FALSE, hyper = "pc", pc.u = 1, pc.alpha = 0.01, pc.u.phi = 0.5, pc.alpha.phi = 2/3, pc.u.cor = 0.7, pc.alpha.cor = 0.9,  pc.st.u = NA, pc.st.alpha = NA, pc.st.slope.u = NA, pc.st.slope.alpha = NA, overdisp.mean = 0, overdisp.prec = 0.4, options = list(config = TRUE), control.inla = list(strategy = "adaptive", int.strategy = "auto"), verbose = FALSE, geo = NULL, rw = NULL, ar = NULL, st.rw = NULL, ...){
+smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")[1], age.groups = c("0", "1-11", "12-23", "24-35", "36-47", "48-59"), age.n = c(1,11,12,12,12,12), age.rw.group = c(1,2,3,3,3,3), age.strata.fixed.group = c(1,2,3,4,5,6), time.model = c("rw1", "rw2", "ar1")[2], st.time.model = NULL, Amat, bias.adj = NULL, bias.adj.by = NULL, formula = NULL, year_label, type.st = 4, survey.effect = FALSE, linear.trend = TRUE, common.trend = FALSE, strata.time.effect = FALSE, hyper = "pc", pc.u = 1, pc.alpha = 0.01, pc.u.phi = 0.5, pc.alpha.phi = 2/3, pc.u.cor = 0.7, pc.alpha.cor = 0.9,  pc.st.u = NA, pc.st.alpha = NA, pc.st.slope.u = NA, pc.st.slope.alpha = NA, overdisp.mean = 0, overdisp.prec = 0.4, options = list(config = TRUE), control.inla = list(strategy = "adaptive", int.strategy = "auto"), verbose = FALSE, geo = NULL, rw = NULL, ar = NULL, st.rw = NULL, ...){
 
   # if(family == "betabinomialna") stop("family = betabinomialna is still experimental.")
   # check region names in Amat is consistent
@@ -252,7 +256,10 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
         stop("Temporal interaction effect only support rw1, rw2, and ar1.")
       }
     }
-
+  }
+  if(time.model == "rw2"){
+    linear.trend = FALSE
+    common.trend = FALSE
   }
   if(!is.temporal){
     message("----------------------------------",
@@ -266,7 +273,15 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
             "\nCluster-level model",
             "\n  Main temporal model:        ", time.model, appendLF = FALSE)
     msg <- paste0(msg, "\nCluster-level model",
-                       "\n  Main temporal model:        ")
+                       "\n  Main temporal model:        ", time.model)
+    if(linear.trend && !common.trend){
+        message("\n  Additional linear trends:   stratum-age-specific", appendLF = FALSE)
+        msg <- paste0(msg, "\n  Additional linear trends:   stratum-age-specific")
+    }
+    if(linear.trend && common.trend){
+        message("\n  Additional linear trends:   shared", appendLF = FALSE)
+        msg <- paste0(msg, "\n  Additional linear trends:   shared")
+    }
   }
 
   
@@ -281,6 +296,9 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
         stop("Row and column names of Amat needs to be the same.")
     }
     is.spatial <- TRUE
+    if(sum(!data$region %in% colnames(Amat)) > 0){
+        stop("Exist regions in the data frame but not in Amat.")
+    }
   }else{
     Amat <- matrix(1,1,1)
     colnames(Amat) <- rownames(Amat) <- "All"
@@ -306,8 +324,8 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
     message("\n  Interaction random slopes:  yes", appendLF=FALSE)
     msg <- paste0(msg, "\n  Interaction random slopes:  yes")
   }else{
-    message("\n", appendLF=FALSE)
-    msg <- paste0(msg, "\n")
+    message("\n  Interaction random slopes:  no", appendLF=FALSE)
+    msg <- paste0(msg, "\n  Interaction random slopes:  no")
   }   
 
 
@@ -315,10 +333,10 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
     age.n <- 1
     age.rw.group <- 1
   }
+  age.strata.fixed.group <- age.strata.fixed.group[1:length(age.n)]
+
   message("\n  Number of age groups: ", length(age.n), appendLF=FALSE)
-  message("\n  Number of age-specific trends per stratum: ", length(unique(age.rw.group)), appendLF=FALSE)
   msg <- paste0(msg, "\n  Number of age groups: ", length(age.n))
-  msg <- paste0(msg, "\n  Number of age-specific trends per stratum: ", length(unique(age.rw.group)))
 
 
   if(is.ar && hyper=="gamma"){
@@ -351,6 +369,7 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
     }
   }
 
+
   has.strata <- TRUE
   if("strata" %in% colnames(data) == FALSE || sum(!is.na(data$strata)) == 0){
     has.strata <- FALSE
@@ -376,6 +395,13 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
     msg <- paste0(msg, "\n  Stratification: yes")
   }
 
+  if(has.strata){
+    message("\n  Number of age-specific fixed effect per stratum: ", length(unique(age.strata.fixed.group)), appendLF=FALSE)
+    msg <- paste0(msg, "\n  Number of age group fixed effect per stratum: ", length(unique(age.strata.fixed.group)))
+    message("\n  Number of age-specific trends per stratum: ", length(unique(age.rw.group)), appendLF=FALSE)
+    msg <- paste0(msg, "\n  Number of age-specific trends per stratum: ", length(unique(age.rw.group)))    
+  }
+
 
   ## Survey fixed effect should only exist if they are nested within the same strata.
   ## This is checked here
@@ -442,8 +468,12 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
   stratalevels <- unique(data$strata)
 
 
-  # If only one strata, this becomes the overall trend
-  if(strata.time.effect){
+age.groups.vec <- age.groups
+data$age.intercept <- data$age
+data$age.orig <- data$age
+
+# If only one strata, this becomes the overall trend
+if(strata.time.effect){
     if(is.null(data$strata)) stop("No strata column in the data.")
     ## Update here age -> age x strata to allow stratum-specific trends
     ## Then redefine age.n and age.rw.group.
@@ -461,12 +491,51 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
     if(is.null(age.groups)){
       age.groups <- stratalevels
       data$age <- data$strata
+      data$age.intercept <- data$strata
     }else{
-      age.groups <- expand.grid(age.groups, stratalevels)
+      age.groups <- expand.grid(age.groups.vec, stratalevels)
       age.groups <- paste(age.groups[,1], age.groups[,2], sep = ":")
       data$age <- paste(data$age, data$strata, sep = ":")
+      # deal with age.intercept in the next section
     }
+  }
 
+  # make new age.intercept variable
+  is.age.diff <- FALSE
+  age.diff.levels <- NULL
+  if(!is.null(age.groups)){
+      # reformat age variable to correspond to intercepts
+      age.groups.new <- NULL
+      for(k in 1:max(age.strata.fixed.group)){
+          age.groups.new <- c(age.groups.new, paste0(age.groups.vec[which(age.strata.fixed.group == k)], collapse = ", "))
+      }
+      # make the list the same length as the original age.groups
+      age.groups.new <- age.groups.new[age.strata.fixed.group]
+
+      # age-stratum effect modeled as difference from baseline?
+      is.age.diff <- ifelse(length(unique(age.strata.fixed.group)) == length(age.strata.fixed.group), 0, 1)
+      if(is.age.diff){
+        # new age.intercept variable as just original age
+        data$age.intercept <- data$age.orig
+        # add new age.diff variable as the difference 
+        data$age.diff <- age.groups.new[match(data$age.orig, age.groups.vec)]
+        data$age.diff <- paste(data$age.diff, data$strata, sep = ":") 
+        ## 
+        ##  This artificial first level (instead of NA) is needed to 
+        ##    make inla specify the model.matrix correctly.
+        ##
+        data$age.diff[data$strata == stratalevels[1]] <- "base"
+        age.diff.levels <- sort(unique(data$age.diff))
+        age.diff.levels <- c("base", age.diff.levels[age.diff.levels != "base"])
+      }else{          
+        # new age.intercept variable as age * strata
+        data$age.intercept <- age.groups.new[match(data$age.orig, age.groups.vec)]
+        data$age.intercept <- paste(data$age.intercept, data$strata, sep = ":") 
+        data$age.diff <- NA
+
+        age.groups.new <- expand.grid(age.groups.new, stratalevels)
+        age.groups.new <- paste(age.groups.new[,1], age.groups.new[,2], sep = ":")
+      }
   }
   if(strata.time.effect){
     message("\n  Strata-specific temporal trends: yes", appendLF=FALSE)
@@ -621,7 +690,7 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
     # exdat <- merge(exdat, cluster.time, by.x = c("cluster", "time.struct"), by.y = c("cluster", "time"))
     # # exdat$nugget.id <- 1:dim(exdat)[1]
 
-    if(is.ar || is.main.ar){
+    if(is.ar || is.main.ar || linear.trend){
       exdat$time.slope <- exdat$time.struct 
       center <- N/2 + 1e-5 # avoid exact zero in the lincomb creation
       exdat$time.slope <- (exdat$time.slope  - center) / ((N - 1))
@@ -674,13 +743,14 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
           }
          
           # AR1 add slope
-           if(is.main.ar && !common.trend){
+           if(linear.trend && !common.trend){
               tmp <- paste(slope.fixed.names, collapse = " + ")
               formula <- as.formula(paste(c(formula, tmp), collapse = "+"))
            }
-           if(is.main.ar && common.trend){
+           if(linear.trend && common.trend){
               formula <- as.formula(paste(c(formula, "time.slope"), collapse = "+"))
            }
+
           # Add IID
           formula <- update(formula, ~. + 
                   f(time.unstruct,model="iid", hyper = hyperpc1, values = 1:N))
@@ -709,11 +779,11 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
              }
 
             # AR1 add slope
-             if(is.main.ar && !common.trend){
+             if(linear.trend && !common.trend){
                 tmp <- paste(slope.fixed.names, collapse = " + ")
                 formula <- as.formula(paste(c(formula, tmp), collapse = "+"))
              }
-             if(is.main.ar && common.trend){
+             if(linear.trend && common.trend){
                 formula <- as.formula(paste(c(formula, "time.slope"), collapse = "+"))
              }
 
@@ -986,12 +1056,23 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
     }
     if(strata.time.effect){
         # In this case, age is age x strata already
-        formula <- update(formula, ~. -1 + age) 
+        if(is.age.diff){
+           formula <- update(formula, ~. -1 + age.intercept + age.diff) 
+         }else{
+            # In this case, age is age x strata already
+            formula <- update(formula, ~. -1 + age.intercept)           
+         } 
     }else{
         if(has.strata){
-          formula <- update(formula, ~. -1 + age + strata)
+          # In this case, age is age x strata already
+          if(is.age.diff){
+             formula <- update(formula, ~. -1 + age.intercept + strata + age.diff) 
+           }else{
+              # In this case, age is age x strata already
+              formula <- update(formula, ~. -1 + age.intercept + strata)
+           } 
         }else{
-          formula <- update(formula, ~. -1 + age) 
+          formula <- update(formula, ~. -1 + age.intercept) 
         }
     }
     if(!is.null(bias.adj)){
@@ -1022,7 +1103,7 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
     tmp$region.struct <- 1:dim(Amat)[1]
     tmp$region_number <- tmp$region.int <- tmp$region.unstruct <- tmp$region.struct
     tmp$region <- colnames(Amat)[tmp$region.struct]
-    created = c("region.struct", "region_number", "region.unstruct", "region.int", "region", "years", "age", "strata")
+    created = c("region.struct", "region_number", "region.unstruct", "region.int", "region", "years", "age", "age.intercept", "age.diff", "strata")
     tmp[, colnames(tmp) %in% created == FALSE] <- NA
     exdat <- rbind(exdat, tmp)
   }
@@ -1053,9 +1134,9 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
     if(dim(tmp)[1] > 0){
         # remove contents in other columns
         if(dim(Amat)[1] != 1){
-          created <- c(created, "region.struct", "region_number", "region.unstruct", "region.int", "region", "time.struct", "time.unstruct", "time.int", "time.area", "years", "age", "strata")
+          created <- c(created, "region.struct", "region_number", "region.unstruct", "region.int", "region", "time.struct", "time.unstruct", "time.int", "time.area", "years", "age", "age.intercept", "age.diff", "strata")
         }else{
-            created <- c(created, "time.struct", "time.unstruct", "time.int",  "years", "age", "strata")
+            created <- c(created, "time.struct", "time.unstruct", "time.int",  "years", "age", "age.intercept", "age.diff", "strata")
         }
         tmp[, colnames(tmp) %in% created == FALSE] <- NA
         exdat <- rbind(exdat, tmp)
@@ -1084,13 +1165,22 @@ smoothCluster <- function(data, X = NULL, family = c("betabinomial", "binomial")
 
   if(has.strata) exdat$strata <- factor(exdat$strata, levels = stratalevels)
   if(!is.null(age.groups)){
-      exdat$age <- factor(exdat$age, levels = age.groups)    
+      exdat$age <- factor(exdat$age, levels = age.groups)
+      if(is.age.diff){
+        exdat$age.intercept <- factor(exdat$age.intercept, levels = unique(age.groups.vec))
+        exdat$age.diff <- factor(exdat$age.diff, levels = age.diff.levels)
+      }else{
+          exdat$age.intercept <- factor(exdat$age.intercept, levels = unique(age.groups.new))
+      }   
+      
   }else{
-     formula <- update(formula, ~.-age)   
+     formula <- update(formula, ~. - age.intercept)  
+     if(is.age.diff)  formula <- update(formula, ~. - age.diff)  
   }
   # if only one level, use the default intercept instead of age
   if(length(age.groups) == 1){
-    formula <- update(formula, ~.-age + 1)   
+    formula <- update(formula, ~. - age.intercept + 1)   
+    if(is.age.diff)  formula <- update(formula, ~. - age.diff)  
   }
   exdat$age.idx <- match(exdat$age, age.groups)
   exdat$age.rep.idx <- age.rw.group[exdat$age.idx]
@@ -1147,7 +1237,7 @@ if(family == "betabinomialna"){
 
  priors <- list(pc.u = pc.u, pc.alpha = pc.alpha, pc.u.phi = pc.u.phi, pc.alpha.phi = pc.alpha.phi, pc.u.cor = pc.u.cor, pc.alpha.cor = pc.alpha.cor,  pc.st.u = pc.st.u, pc.st.alpha = pc.st.alpha, pc.st.slope.u = pc.st.slope.u, pc.st.slope.prec.u = pc.st.slope.prec.u, pc.st.slope.alpha = pc.st.slope.alpha, overdisp.mean = overdisp.mean, overdisp.prec = overdisp.prec)
 
- out <- list(model = formula, fit = fit, family= family, Amat = Amat, newdata = exdat, time = seq(0, N - 1), area = seq(0, region_count - 1), time.area = time.area, survey.table = survey.table, is.yearly = FALSE, type.st = type.st, year_label = year_label, age.groups = age.groups, age.n = age.n, age.rw.group = age.rw.group, strata.base = strata.base, rw = rw, ar = ar, strata.time.effect = strata.time.effect,  priors = priors, year_range = NA, Amat = Amat, has.Amat = TRUE, is.temporal = is.temporal, covariate.names = covariate.names, msg = msg)
+ out <- list(model = formula, fit = fit, family= family, Amat = Amat, newdata = exdat, time = seq(0, N - 1), area = seq(0, region_count - 1), time.area = time.area, survey.table = survey.table, is.yearly = FALSE, type.st = type.st, year_label = year_label, age.groups = age.groups, age.groups.new = age.groups.new, age.n = age.n, age.rw.group = age.rw.group, age.strata.fixed.group = age.strata.fixed.group, strata.base = strata.base, rw = rw, ar = ar, strata.time.effect = strata.time.effect,  priors = priors, year_range = NA, Amat = Amat, has.Amat = TRUE, is.temporal = is.temporal, covariate.names = covariate.names, msg = msg)
  class(out) <- "SUMMERmodel"
  return(out)
     
